@@ -1,13 +1,13 @@
 ;;; vertico-repeat.el --- Repeat Vertico sessions -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 1.9
-;; Package-Requires: ((emacs "27.1") (compat "30") (vertico "1.9"))
-;; Homepage: https://github.com/minad/vertico
+;; Version: 2.1
+;; Package-Requires: ((emacs "28.1") (compat "30") (vertico "2.1"))
+;; URL: https://github.com/minad/vertico
 
 ;; This file is part of GNU Emacs.
 
@@ -35,8 +35,7 @@
 ;;
 ;; It is necessary to register a minibuffer setup hook, which saves
 ;; the Vertico state for repetition.  In order to save the history
-;; across Emacs sessions, enable `savehist-mode' and add
-;; `vertico-repeat-history' to `savehist-additional-variables'.
+;; across Emacs sessions, enable `savehist-mode'.
 ;;
 ;; (keymap-global-set "M-R" #'vertico-repeat)
 ;; (keymap-set vertico-map "M-P" #'vertico-repeat-previous)
@@ -66,12 +65,12 @@
 
 (defcustom vertico-repeat-transformers
   (list #'vertico-repeat--filter-empty
-        #'vertico-repeat--filter-commands)
+        #'vertico-repeat--filter-commands
+        #'vertico-repeat--remove-long)
   "List of functions to apply to history element before saving."
   :type '(repeat function)
   :group 'vertico)
 
-(declare-function vertico-multiform-vertical "ext:vertico-multiform")
 (defvar vertico-multiform--display-modes)
 (defvar vertico-repeat-history nil)
 (defvar-local vertico-repeat--command nil)
@@ -87,6 +86,13 @@
   "Filter SESSION if input is empty."
   (and (cadr session) (not (equal (cadr session) "")) session))
 
+(defun vertico-repeat--remove-long (session)
+  "Remove overly long candidate from SESSION."
+  (when-let ((cand (caddr session))
+             ((and (stringp cand) (length> cand 200))))
+    (setf (cddr session) (cdddr session)))
+  session)
+
 (defun vertico-repeat--save-input ()
   "Save current minibuffer input."
   (setq vertico-repeat--input (minibuffer-contents-no-properties)))
@@ -110,6 +116,10 @@
         (transform vertico-repeat-transformers))
     (while (and transform (setq session (funcall (pop transform) session))))
     (when session
+      (unless (or (not (bound-and-true-p savehist-mode))
+                  (memq 'vertico-repeat-history (bound-and-true-p savehist-ignored-variables)))
+        (defvar savehist-minibuffer-history-variables)
+        (add-to-list 'savehist-minibuffer-history-variables 'vertico-repeat-history))
       (add-to-history 'vertico-repeat-history session))))
 
 (defun vertico-repeat--restore (session)
@@ -128,6 +138,7 @@
              (mode (seq-find #'symbolp (cddr session)))
              ((bound-and-true-p vertico-multiform-mode))
              ((not (and (boundp mode) (symbol-value mode)))))
+    (declare-function vertico-multiform-vertical "ext:vertico-multiform")
     (vertico-multiform-vertical mode))
   (vertico--exhibit))
 
@@ -180,9 +191,6 @@ selected candidate for the current command."
        (setq vertico-repeat--pos n)
        session))))
 
-(define-obsolete-function-alias
-  'vertico-repeat-last 'vertico-repeat-previous "1.4")
-
 ;;;###autoload
 (defun vertico-repeat-select ()
   "Select a Vertico session from the session history and repeat it.
@@ -222,6 +230,7 @@ previous sessions for the current command."
                   (if current-cmd
                       (format "History of %s: " current-cmd)
                     "Completion history: ")
+                  ;; TODO: Use `completion-table-with-metadata'
                   (lambda (str pred action)
                     (if (eq action 'metadata)
                         '(metadata (display-sort-function . identity)
