@@ -1,13 +1,13 @@
 ;;; vertico-reverse.el --- Reverse the Vertico display -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 1.9
-;; Package-Requires: ((emacs "27.1") (compat "30") (vertico "1.9"))
-;; Homepage: https://github.com/minad/vertico
+;; Version: 2.1
+;; Package-Requires: ((emacs "28.1") (compat "30") (vertico "2.1"))
+;; URL: https://github.com/minad/vertico
 
 ;; This file is part of GNU Emacs.
 
@@ -29,10 +29,10 @@
 ;; This package is a Vertico extension, which reverses the list of
 ;; candidates.
 ;;
-;; The mode can be enabled globally or via `vertico-multiform-mode'
-;; per command or completion category.  Alternatively the reverse
-;; display can be toggled temporarily with M-R if
-;; `vertico-multiform-mode' is enabled.
+;; The mode `vertico-reverse-mode' can be enabled globally or via
+;; `vertico-multiform-mode' per command or completion category.
+;; Alternatively the reverse display can be toggled temporarily
+;; with M-R if `vertico-multiform-mode' is enabled.
 
 ;;; Code:
 
@@ -73,8 +73,7 @@
   (let ((string (apply #'concat lines)))
     (add-face-text-property 0 (length string) 'default 'append string)
     (overlay-put vertico--candidates-ov 'before-string string)
-    (overlay-put vertico--candidates-ov 'after-string nil))
-  (vertico--resize-window (length lines)))
+    (overlay-put vertico--candidates-ov 'after-string nil)))
 
 (provide 'vertico-reverse)
 ;;; vertico-reverse.el ends here
