@@ -1,13 +1,13 @@
 ;;; vertico.el --- VERTical Interactive COmpletion -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 1.9
-;; Package-Requires: ((emacs "27.1") (compat "30"))
-;; Homepage: https://github.com/minad/vertico
+;; Version: 2.1
+;; Package-Requires: ((emacs "28.1") (compat "30"))
+;; URL: https://github.com/minad/vertico
 ;; Keywords: convenience, files, matching, completion
 
 ;; This file is part of GNU Emacs.
@@ -42,7 +42,8 @@
 (defgroup vertico nil
   "VERTical Interactive COmpletion."
   :link '(info-link :tag "Info Manual" "(vertico)")
-  :link '(url-link :tag "Homepage" "https://github.com/minad/vertico")
+  :link '(url-link :tag "Website" "https://github.com/minad/vertico")
+  :link '(url-link :tag "Wiki" "https://github.com/minad/vertico/wiki")
   :link '(emacs-library-link :tag "Library Source" "vertico.el")
   :group 'convenience
   :group 'minibuffer
@@ -91,14 +92,15 @@ The value should lie between 0 and vertico-count/2."
   "Replacements for multiline strings."
   :type '(cons (string :tag "Newline") (string :tag "Truncation")))
 
-(defcustom vertico-sort-function #'vertico-sort-history-length-alpha
+(defcustom vertico-sort-function
+  (and (fboundp 'vertico-sort-history-length-alpha) 'vertico-sort-history-length-alpha)
   "Default sorting function, used if no `display-sort-function' is specified."
-  :type `(choice
+  :type '(choice
           (const :tag "No sorting" nil)
-          (const :tag "By history, length and alpha" ,#'vertico-sort-history-length-alpha)
-          (const :tag "By history and alpha" ,#'vertico-sort-history-alpha)
-          (const :tag "By length and alpha" ,#'vertico-sort-length-alpha)
-          (const :tag "Alphabetically" ,#'vertico-sort-alpha)
+          (const :tag "By history, length and alpha" vertico-sort-history-length-alpha)
+          (const :tag "By history and alpha" vertico-sort-history-alpha)
+          (const :tag "By length and alpha" vertico-sort-length-alpha)
+          (const :tag "Alphabetically" vertico-sort-alpha)
           (function :tag "Custom function")))
 
 (defcustom vertico-sort-override-function nil
@@ -116,7 +118,7 @@ The value should lie between 0 and vertico-count/2."
 (defface vertico-group-title '((t :inherit shadow :slant italic))
   "Face used for the title text of the candidate group headlines.")
 
-(defface vertico-group-separator '((t :inherit shadow :strike-through t))
+(defface vertico-group-separator '((t :inherit vertico-group-title :strike-through t))
   "Face used for the separator lines of the candidate groups.")
 
 (defface vertico-current '((t :inherit highlight :extend t))
@@ -139,14 +141,12 @@ The value should lie between 0 and vertico-count/2."
   "<remap> <exit-minibuffer>" #'vertico-exit
   "<remap> <kill-ring-save>" #'vertico-save
   "M-RET" #'vertico-exit-input
-  "TAB" #'vertico-insert)
+  "TAB" #'vertico-insert
+  "<touchscreen-begin>" #'ignore)
 
 (defvar-local vertico--hilit #'identity
   "Lazy candidate highlighting function.")
 
-(defvar-local vertico--history-hash nil
-  "History hash table and corresponding base string.")
-
 (defvar-local vertico--candidates-ov nil
   "Overlay showing the candidates.")
 
@@ -189,65 +189,6 @@ The value should lie between 0 and vertico-count/2."
 (defvar-local vertico--allow-prompt nil
   "Prompt selection is allowed.")
 
-(defun vertico--history-hash ()
-  "Recompute history hash table and return it."
-  (or (and (equal (car vertico--history-hash) vertico--base) (cdr vertico--history-hash))
-      (let* ((base vertico--base)
-             (base-len (length base))
-             (hist (and (not (eq minibuffer-history-variable t)) ;; Disabled for `t'.
-                        (symbol-value minibuffer-history-variable)))
-             (hash (make-hash-table :test #'equal :size (length hist)))
-             (file-p (and (> base-len 0) ;; Step-wise completion, unlike `project-find-file'
-                          (eq minibuffer-history-variable 'file-name-history)))
-             (curr-file (when-let ((win (and file-p (minibuffer-selected-window)))
-                                   (file (buffer-file-name (window-buffer win))))
-                          (abbreviate-file-name file))))
-        (cl-loop for elem in hist for index from 0 do
-                 (when (and (not (equal curr-file elem)) ;; Deprioritize current file
-                            (or (= base-len 0)
-                                (and (>= (length elem) base-len)
-                                     (eq t (compare-strings base 0 base-len elem 0 base-len)))))
-                   (let ((file-sep (and file-p (string-search "/" elem base-len))))
-                     ;; Drop base string from history elements & special file handling.
-                     (when (or (> base-len 0) file-sep)
-                       (setq elem (substring elem base-len (and file-sep (1+ file-sep)))))
-                     (unless (gethash elem hash) (puthash elem index hash)))))
-        (cdr (setq vertico--history-hash (cons base hash))))))
-
-(defun vertico--length-string< (x y)
-  "Sorting predicate which compares X and Y first by length then by `string<'."
-  (or (< (length x) (length y)) (and (= (length x) (length y)) (string< x y))))
-
-(defun vertico--sort-decorated (list)
-  "Sort decorated LIST and remove decorations."
-  (setq list (sort list #'car-less-than-car))
-  (cl-loop for item on list do (setcar item (cdar item)))
-  list)
-
-(defmacro vertico--define-sort (by bsize bindex bpred pred)
-  "Generate optimized sorting function.
-The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
-  `(defun ,(intern (mapconcat #'symbol-name `(vertico sort ,@by) "-")) (candidates)
-     ,(concat "Sort candidates by " (mapconcat #'symbol-name by ", ") ".")
-     (let* ((buckets (make-vector ,bsize nil))
-            ,@(and (eq (car by) 'history) '((hhash (vertico--history-hash)) (hcands))))
-       (dolist (% candidates)
-         ;; Find recent candidate in history or fill bucket
-         (,@(if (not (eq (car by) 'history)) `(progn)
-              `(if-let ((idx (gethash % hhash))) (push (cons idx %) hcands)))
-          (let ((idx (min ,(1- bsize) ,bindex)))
-            (aset buckets idx (cons % (aref buckets idx))))))
-       (nconc ,@(and (eq (car by) 'history) '((vertico--sort-decorated hcands)))
-              (mapcan (lambda (bucket) (sort bucket #',bpred))
-                      (nbutlast (append buckets nil)))
-              ;; Last bucket needs special treatment
-              (sort (aref buckets ,(1- bsize)) #',pred)))))
-
-(vertico--define-sort (history length alpha) 32 (length %) string< vertico--length-string<)
-(vertico--define-sort (history alpha) 32 (if (equal % "") 0 (/ (aref % 0) 4)) string< string<)
-(vertico--define-sort (length alpha) 32 (length %) string< vertico--length-string<)
-(vertico--define-sort (alpha) 32 (if (equal % "") 0 (/ (aref % 0) 4)) string< string<)
-
 (defun vertico--affixate (cands)
   "Annotate CANDS with annotation function."
   (if-let ((aff (vertico--metadata-get 'affixation-function)))
@@ -315,13 +256,13 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
                (before (substring content 0 pt))
                (after (substring content pt))
                ;; bug#47678: `completion-boundaries' fails for `partial-completion'
-               ;; if the cursor is moved between the slashes of "~//".
+               ;; if the cursor is moved before the slashes of "~//".
                ;; See also corfu.el which has the same issue.
                (bounds (condition-case nil
                            (completion-boundaries before table pred after)
                          (t (cons 0 (length after)))))
                (field (substring content (car bounds) (+ pt (cdr bounds))))
-               ;; `minibuffer-completing-file-name' has been obsoleted by the completion category
+               ;; bug#75910: category instead of `minibuffer-completing-file-name'
                (completing-file (eq 'file (vertico--metadata-get 'category)))
                (`(,all . ,hl) (vertico--filter-completions content table pred pt vertico--metadata))
                (base (or (when-let ((z (last all))) (prog1 (cdr z) (setcdr z nil))) 0))
@@ -365,6 +306,12 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
                                           (test-completion content table pred)))
                                  -1 0))))))
 
+(defun vertico--hilit (cand)
+  "Highlight CAND string with lazy highlighting."
+  ;; bug#77754: Highlight unquoted string.
+  (funcall vertico--hilit (substring (or (get-text-property
+                                          0 'completion--unquoted cand) cand))))
+
 (defun vertico--cycle (list n)
   "Rotate LIST to position N."
   (nconc (copy-sequence (nthcdr n list)) (seq-take list n)))
@@ -478,11 +425,12 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
 
 (defun vertico--format-group-title (title cand)
   "Format group TITLE given the current CAND."
+  ;; Copy candidate highlighting if title is a prefix of the candidate.
   (when (string-prefix-p title cand)
-    ;; Highlight title if title is a prefix of the candidate
-    (setq cand (propertize cand 'face 'vertico-group-title)
-          title (substring (funcall vertico--hilit cand) 0 (length title)))
+    (setq title (substring cand 0 (length title)))
     (vertico--remove-face 0 (length title) 'completions-first-difference title))
+  (setq title (substring title))
+  (add-face-text-property 0 (length title) 'vertico-group-title t title)
   (format (concat vertico-group-format "\n") title))
 
 (defun vertico--format-count ()
@@ -521,7 +469,8 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
     (vertico--update 'interruptible)
     (vertico--prompt-selection)
     (vertico--display-count)
-    (vertico--display-candidates (vertico--arrange-candidates))))
+    (vertico--display-candidates (vertico--arrange-candidates))
+    (vertico--resize)))
 
 (defun vertico--goto (index)
   "Go to candidate with INDEX."
@@ -532,7 +481,7 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
 
 (defun vertico--candidate (&optional hl)
   "Return current candidate string with optional highlighting if HL is non-nil."
-  (let ((content (substring (or (car-safe vertico--input) (minibuffer-contents-no-properties)))))
+  (let ((content (or (car-safe vertico--input) (minibuffer-contents-no-properties))))
     (cond
      ((>= vertico--index 0)
       (let ((cand (substring (nth vertico--index vertico--candidates))))
@@ -540,7 +489,7 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
         ;; `completion--twq-all' hack.  This should better be fixed in Emacs
         ;; itself, the corresponding code is already marked as fixme.
         (vertico--remove-face 0 (length cand) 'completions-common-part cand)
-        (concat vertico--base (if hl (funcall vertico--hilit cand) cand))))
+        (concat vertico--base (if hl (vertico--hilit cand) cand))))
      ((and (equal content "") (or (car-safe minibuffer-default) minibuffer-default)))
      (t content))))
 
@@ -549,8 +498,8 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
   (let ((rm minibuffer--require-match))
     (or (memq rm '(nil confirm-after-completion))
         (equal "" input) ;; Null completion, returns default value
-        (and (functionp rm) (funcall rm input)) ;; Emacs 29 supports functions
-        (test-completion input minibuffer-completion-table minibuffer-completion-predicate)
+        (if (functionp rm) (funcall rm input) ;; Emacs 29 supports functions
+          (test-completion input minibuffer-completion-table minibuffer-completion-predicate))
         (if (eq rm 'confirm) (eq (ignore-errors (read-char "Confirm")) 13)
           (minibuffer-message "Match required") nil))))
 
@@ -571,7 +520,7 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
            (candidates
             (vertico--affixate
              (cl-loop repeat vertico-count for c in (nthcdr index vertico--candidates)
-                      collect (funcall vertico--hilit (substring c))))))
+                      collect (vertico--hilit c)))))
       (pcase-dolist ((and cand `(,str . ,_)) candidates)
         (when-let ((new-title (and group-fun (funcall group-fun str nil))))
           (unless (equal title new-title)
@@ -603,31 +552,31 @@ The function is configured by BY, BSIZE, BINDEX, BPRED and PRED."
   "Update candidates overlay `vertico--candidates-ov' with LINES."
   (move-overlay vertico--candidates-ov (point-max) (point-max))
   (overlay-put vertico--candidates-ov 'after-string
-               (apply #'concat #(" " 0 1 (cursor t)) (and lines "\n") lines))
-  (vertico--resize-window (length lines)))
+               (apply #'concat #(" " 0 1 (cursor t)) (and lines "\n") lines)))
 
-(cl-defgeneric vertico--resize-window (height)
-  "Resize active minibuffer window to HEIGHT."
+(cl-defgeneric vertico--resize ()
+  "Resize active minibuffer window."
   (setq-local truncate-lines (< (point) (* 0.8 (vertico--window-width)))
-              resize-mini-windows 'grow-only
+              resize-mini-windows vertico-resize
               max-mini-window-height 1.0)
   (unless truncate-lines (set-window-hscroll nil 0))
-  (unless (frame-root-window-p (active-minibuffer-window))
-    (unless vertico-resize (setq height (max height vertico-count)))
-    (let ((dp (- (max (cdr (window-text-pixel-size))
-                      (* (default-line-height) (1+ height)))
+  (unless (or vertico-resize (frame-root-window-p (active-minibuffer-window)))
+    (let ((delta (- (max (cdr (window-text-pixel-size))
+                         (* (default-line-height) (1+ vertico-count)))
                  (window-pixel-height))))
-      (when (or (and (> dp 0) (/= height 0))
-                (and (< dp 0) (eq vertico-resize t)))
-        (window-resize nil dp nil nil 'pixelwise)))))
+      (when (/= 0 delta) (window-resize nil delta nil nil 'pixelwise)))))
 
 (cl-defgeneric vertico--prepare ()
   "Ensure that the state is prepared before running the next command."
-  (when (and (symbolp this-command) (string-prefix-p "vertico-" (symbol-name this-command)))
+  (when-let ((cmd (and (symbolp this-command) (symbol-name this-command)))
+             ((string-prefix-p "vertico-" cmd))
+             ((not (and vertico--metadata (string-prefix-p "vertico-directory-" cmd)))))
     (vertico--update)))
 
 (cl-defgeneric vertico--setup ()
   "Setup completion UI."
+  (when (boundp 'pixel-scroll-precision-mode)
+    (setq-local pixel-scroll-precision-mode nil))
   (setq-local scroll-margin 0
               vertico--input t
               completion-auto-help nil
@@ -720,7 +669,7 @@ When the prefix argument is 0, the group order is reset."
   (interactive)
   (if (or (use-region-p) (not transient-mark-mode))
       (call-interactively #'kill-ring-save)
-    (kill-new (vertico--candidate))))
+    (kill-new (substring-no-properties (vertico--candidate)))))
 
 (defun vertico-insert ()
   "Insert current candidate in minibuffer."
@@ -747,14 +696,14 @@ When the prefix argument is 0, the group order is reset."
   "Return non-nil if Vertico is active in BUFFER."
   (buffer-local-value 'vertico--input buffer))
 
-;; Emacs 28: Do not show Vertico commands in M-X
-(dolist (sym '(vertico-next vertico-next-group vertico-previous vertico-previous-group
-               vertico-scroll-down vertico-scroll-up vertico-exit vertico-insert
-               vertico-exit-input vertico-save vertico-first vertico-last
-               vertico-repeat-previous ;; autoloads in vertico-repeat.el
-               vertico-quick-jump vertico-quick-exit vertico-quick-insert ;; autoloads in vertico-quick.el
-               vertico-directory-up vertico-directory-enter ;; autoloads in vertico-directory.el
-               vertico-directory-delete-char vertico-directory-delete-word))
+;; Do not show Vertico commands in M-X
+(dolist (sym '( vertico-next vertico-next-group vertico-previous vertico-previous-group
+                vertico-scroll-down vertico-scroll-up vertico-exit vertico-insert
+                vertico-exit-input vertico-save vertico-first vertico-last
+                vertico-repeat-next ;; autoloads in vertico-repeat.el
+                vertico-quick-jump vertico-quick-exit vertico-quick-insert ;; autoloads in vertico-quick.el
+                vertico-directory-up vertico-directory-enter ;; autoloads in vertico-directory.el
+                vertico-directory-delete-char vertico-directory-delete-word))
   (put sym 'completion-predicate #'vertico--command-p))
 
 (provide 'vertico)
